/**
 * @file   get_param.c
 * @author Clement Moreau <clement.moreau@inventel.fr>, Xavier Debreuil <xavier.debreuil@inventel.fr>
 * @date   Sat Mar 29 17:09:22 2003
 * 
 * @brief  rcupration des paramtres d'usine dans le psi/nvram ou dans le block RedBoot config
 * 
 * 
 */

#include <stdio.h>
#include "hard_api/board_api.h"
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

//#define DEBUG
/* for debugging purposes only */
#ifdef DEBUG
#define diag_printf printf
#else
#define diag_printf(fmt,args...)
#endif


#define STR_LENGTH 512
#define SYSTEM_STR0 "redboot_fconfig_get --parameter "
#define SYSTEM_STR1 " /dev/"
#define MAC_BASE_PARAMETER_STR "eth0_esa"
#define WEP_KEY_PARAMETER_STR "wep_key"
#define SERIAL_NUMBER_PARAMETER_STR "serial_number"
#define INVENTEL_WIRELESS_MAGIC_PARAMETER_STR "inventel_wireless_magic"
#define H235_KEY_PARAMETER_STR "h235_key"
#define BT_PIN_PARAMETER_STR "bt_pin"

unsigned char wep_key[13] = {0xFF,0xFF,0xFF,
			     0xFF,0xFF,0xFF,
			     0xFF,0xFF,0xFF,
			     0xFF,0xFF,0xFF,
			     0xFF};
unsigned char wep_key_str[256] = "FFFFFFFFFFFFFFFFFFFFFFFFFF";
unsigned char serial_number_str[256] = "0000000000000000";
unsigned char inventel_wireless_magic_str[256] = "0xFFFFFFFF";
unsigned char h235_key_str[256] = "1234567812345678";
unsigned char mac_base[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
unsigned char mac_base_str[64] = "00:07:3A:FF:FF:FF";
unsigned char boot_loader_build_id[256] = "CFE";
unsigned char bt_pin_str[18] = "0000";


void get_param_exit(void)
{
  printf("MAC_BASE=%s\nWEP_KEY=%s\nBOOT_LOADER_BUILD_ID=\"%s\"\nBL_SERIAL_NUMBER=%s\nBL_INVENTEL_WIRELESS_MAGIC=%s\nBT_PIN=%s\n",
	 mac_base_str, wep_key_str, boot_loader_build_id, serial_number_str, inventel_wireless_magic_str,bt_pin_str);
  exit(0);
}


int get_param_main()
{
  FILE *fptr;
  unsigned char proc_mtd_str[STR_LENGTH], redboot_config_mtd_str[STR_LENGTH],
   system_str[STR_LENGTH], output[STR_LENGTH];

  unsigned char *ptr, *ptr0;
  int i ;

  //get_param_exit();

  for (i = 0 ; i < STR_LENGTH ; i ++) {
    proc_mtd_str[i] = 0;
    redboot_config_mtd_str[i] = 0;
  }

  fptr = fopen("/proc/mtd", "r");
  if(!fptr) {
    printf("Unable to open /proc/mtd\n");
    get_param_exit();
  }
  else {
    fread((void *)proc_mtd_str, sizeof(unsigned char), STR_LENGTH, fptr);
    diag_printf("%s\n", proc_mtd_str);
  }
  fclose(fptr);

  // look for RedBoot mtd partion
  ptr = strstr(proc_mtd_str, "RedBoot");

  if(!ptr) {
    diag_printf("CFE bootloader\n");
    //wsysGetMacAddress(mac_base,1);
    sprintf(mac_base_str, "%02x:%02x:%02x:%02x:%02x:%02x\n", mac_base[0],
	    mac_base[1], mac_base[2], mac_base[3], mac_base[4], mac_base[5]);
    //wsysGetWepKey(wep_key,1);
    sprintf(wep_key_str, "%02X%02X%02X%02X\
%02X%02X%02X%02X\
%02X%02X%02X%02X\
%02X\n",
	   wep_key[0],
	   wep_key[1],
	   wep_key[2],
	   wep_key[3],
	   wep_key[4],
	   wep_key[5],
	   wep_key[6],
	   wep_key[7],
	   wep_key[8],
	   wep_key[9],
	   wep_key[10],
	   wep_key[11],
	   wep_key[12]);
    //wsysGetBuildId(boot_loader_build_id, 1);
    get_param_exit();
  }
  else {
    diag_printf("RedBoot bootloader\n");
    strcpy(boot_loader_build_id, "RedBoot");
    diag_printf("%s\n", ptr);
    
    // look for the number of RedBoot config mtd partition
    ptr = strstr(proc_mtd_str, "RedBoot config");
    
    // RedBoot config partition is never the first partition :
    // there is at least one partition (RedBoot) before
    do {
      ptr --;
    } while (*ptr != '\n');
    ptr ++;
    // ptr points to : mtd7: 00001000 00010000 "RedBoot config"

    ptr0 = redboot_config_mtd_str;
    while (*ptr != ':')
      *ptr0 ++ = *ptr ++;
    *ptr0 = 0;

    diag_printf("MTD RedBoot config partition : %s\n", redboot_config_mtd_str);

    strcpy(system_str, SYSTEM_STR0);
    strcat(system_str, MAC_BASE_PARAMETER_STR);
    strcat(system_str, SYSTEM_STR1);
    strcat(system_str, redboot_config_mtd_str);
    diag_printf("%s\n", system_str);

#ifdef DEBUG
    sleep(1);
#endif
    
    fptr = popen(system_str, "r");
    if (fptr == NULL) {
      diag_printf("Error popen\n");
      get_param_exit();
    }
    
    if (fread(output, sizeof(unsigned char), STR_LENGTH, fptr) < 17)
      get_param_exit();

    output[17] = 0;
    strcpy(mac_base_str, output);
    pclose(fptr);
    diag_printf("MAC_BASE=%s\n", mac_base_str);
    //system(system_str);


    strcpy(system_str, SYSTEM_STR0);
    strcat(system_str, WEP_KEY_PARAMETER_STR);
    strcat(system_str, SYSTEM_STR1);
    strcat(system_str, redboot_config_mtd_str);
    diag_printf("%s\n", system_str);

    fptr = popen(system_str, "r");
    if (fptr == NULL) {
      diag_printf("Error popen\n");
      get_param_exit();
    }
    
    if (fread(output, sizeof(unsigned char), STR_LENGTH, fptr) < 26)
      get_param_exit();

    output[26] = 0;
    strcpy(wep_key_str, output);
    pclose(fptr);
    diag_printf("WEP_KEY=%s\n", wep_key_str);
    //system(system_str);


    strcpy(system_str, SYSTEM_STR0);
    strcat(system_str, SERIAL_NUMBER_PARAMETER_STR);
    strcat(system_str, SYSTEM_STR1);
    strcat(system_str, redboot_config_mtd_str);
    diag_printf("%s\n", system_str);

    fptr = popen(system_str, "r");
    if (fptr == NULL) {
      diag_printf("Error popen\n");
      get_param_exit();
    }
    
    if (fread(output, sizeof(unsigned char), STR_LENGTH, fptr) < 16)
      get_param_exit();

    output[16] = 0;
    strcpy(serial_number_str, output);
    pclose(fptr);
    diag_printf("BL_SERIAL_NUMBER=%s\n", serial_number_str);


    strcpy(system_str, SYSTEM_STR0);
    strcat(system_str, INVENTEL_WIRELESS_MAGIC_PARAMETER_STR);
    strcat(system_str, SYSTEM_STR1);
    strcat(system_str, redboot_config_mtd_str);
    diag_printf("%s\n", system_str);

    fptr = popen(system_str, "r");
    if (fptr == NULL) {
      diag_printf("Error popen\n");
      get_param_exit();
    }
    
    if (fread(output, sizeof(unsigned char), STR_LENGTH, fptr) < 10)
      get_param_exit();

    output[10] = 0;
    strcpy(inventel_wireless_magic_str, output);
    pclose(fptr);
    diag_printf("BL_INVENTEL_WIRELESS_MAGIC=%s\n", inventel_wireless_magic_str);


    strcpy(system_str, SYSTEM_STR0);
    strcat(system_str, BT_PIN_PARAMETER_STR);
    strcat(system_str, SYSTEM_STR1);
    strcat(system_str, redboot_config_mtd_str);
    diag_printf("%s\n", system_str);

    fptr = popen(system_str, "r");
    if (fptr == NULL) {
      diag_printf("Error popen\n");
      get_param_exit();
    }
    
    memset( output, 0, sizeof( output ));
    if (fread(output, sizeof(unsigned char), STR_LENGTH, fptr) < 1)
      get_param_exit();

    output[16] = 0;  //on limite la taille a 16 char dans tous les cas
    strcpy(bt_pin_str, output);
    pclose(fptr);
    diag_printf("BT_PIN=%s\n", bt_pin_str);


//bfo
#if 0
    strcpy(system_str, SYSTEM_STR0);
    strcat(system_str, H235_KEY_PARAMETER_STR);
    strcat(system_str, SYSTEM_STR1);
    strcat(system_str, redboot_config_mtd_str);
    diag_printf("%s\n", system_str);

    fptr = popen(system_str, "r");
    if (fptr == NULL) {
      diag_printf("Error popen\n");
      get_param_exit();
    }
    
    if (fread(output, sizeof(unsigned char), STR_LENGTH, fptr) < 16)
      get_param_exit();

    output[16] = 0;
    strcpy(h235_key_str, output);
    pclose(fptr);
    diag_printf("BL_H235_KEY=%s\n", h235_key_str);
    //system(system_str);
#endif


    get_param_exit();
  }

  return 0;
}
